/*
 * Project: VizKit
 * Version: 3.3.4
 
 * Date: 20130804
 * File: TrackLyrics.h
 *
 */

/***************************************************************************

Copyright (c) 2004-2013 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#ifndef TrackLyrics_h
#define TrackLyrics_h


#include "VisualTypes.h"
#include "VisualTextAutoScrollView.h"
#include "VisualItemIdentifier.h"
#include "VisualStringStyle.h"


namespace VizKit {

	class VisualString; // Forward declaration (to avoid include of header file).

	/**
	 * Shows the lyrics of the current audio track.
	 * TrackLyrics performs the action of the TrackLyricsActor.
	 */
	class TrackLyrics {

	public:

		/**
		 * The constructor.
		 */
		TrackLyrics();

		/**
		 * The destructor.
		 */
		~TrackLyrics();

		/**
		 * Shows the track lyrics action.
         * @param anElapsedAudioTime The elapsed audio time.
         * @param aRemainingAudioTime The remaining audio time.
		 */
		void show(uint32 anElapsedAudioTime, uint32 aRemainingAudioTime);

		/**
		 * Returns the current string style of the track lyrics string.
		 * @return The current string style of the track lyrics string.
		 */
		VisualStringStyle getTrackLyricsStringStyle(void);
        
        /**
		 * Sets the track lyrics string.
		 * @param aTrackLyricsString The styled track lyrics string.
		 */
        void setTrackLyricsString(VisualStyledString &aTrackLyricsString);
        
        /**
		 * Refreshes the layout based on current canvas size.
		 */
		void refresh(void);

		/**
		 * Starts fading in the texture of the track lyrics.
		 * @param durationInMilliseconds The duration of the animation measured in milliseconds.
         * @param startWithCurrentOpacityValue If yes, the animation starts with the current opacity value. If no, the animation starts with 0 opacity.
		 */
		void fadeIn(uint32 durationInMilliseconds, bool startWithCurrentOpacityValue);

		/**
		 * Starts fading out the texture of the track lyrics.
		 * @param durationInMilliseconds The duration of the animation measured in milliseconds.
		 * @param stopValue The stop value of the fade out process (default = 0.0f).
		 */
		void fadeOut(uint32 durationInMilliseconds, double stopValue = 0.0);

		/**
		 * Clears the memory allocated for the track lyrics action.
		 */
		void clear(void);

	private:

		/** The VisualTextAutoScrollView of the track lyrics action. */
		VisualTextAutoScrollView *trackLyricsAutoScrollView;
		
		/** Identifier of the vertex chain (the vertices used for display of the track lyrics string texture). */
		VisualItemIdentifier vertexChainId;
		
        /** Identifier of the fade in and fade out animation. */
        VisualItemIdentifier opacityAnimationIdentifier;
        
        /** Callback of the opacity animation. */
        static void animateOpacity(double currentPosition, void* userData);
	};

}

#endif /* TrackLyrics_h */
