/*
 * Project: VizKit
 * Version: 3.3.4
 
 * Date: 20130804
 * File: VisualThreadingManager.cpp
 *
 */

/***************************************************************************

Copyright (c) 2004-2013 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#include "VisualThreadingManager.h"
#include "VisualErrorHandling.h"
#include "VisualTiming.h"


#if TARGET_OS_MAC
#include <CoreServices/../Frameworks/CarbonCore.framework/Headers/Debugging.h> // GetMacOSStatusErrorString
#endif


using namespace VizKit;



VisualThreadingManager* VisualThreadingManager::theVisualThreadingManager = NULL;


VisualThreadingManager::VisualThreadingManager() {
	terminateAllThreads = false;
#if TARGET_OS_MAC
	notificationQueue = NULL;
	OSStatus status = noErr;
	status = MPCreateQueue(&notificationQueue);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for notificationQueue in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
	status = MPCreateCriticalRegion(&downloadQueueSingletonCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for downloadQueueSingletonCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
	status = MPCreateCriticalRegion(&downloadQueueAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for downloadQueueAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
	status = MPCreateCriticalRegion(&downloadQueueAddRequestCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for downloadQueueAddRequestCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
	status = MPCreateCriticalRegion(&downloadQueueStopRequestCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for downloadQueueStopRequestCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
	status = MPCreateCriticalRegion(&notificationQueueSingletonCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for notificationQueueSingletonCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
	status = MPCreateCriticalRegion(&notificationQueueAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for notificationQueueAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
	status = MPCreateCriticalRegion(&inputNotificationQueueAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for inputNotificationQueueAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
	status = MPCreateCriticalRegion(&timeStoreSingletonCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for timeStoreSingletonCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
	status = MPCreateCriticalRegion(&timeStoreAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for timeStoreAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&currentTimeAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for currentTimeAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&appProcAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for appProcAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&appCookieAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for appCookieAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&appVersionAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for appVersionAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&visualizerAPIVersionAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for visualizerAPIVersionAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&trackPlayPositionAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for trackPlayPositionAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&audioDataTimestampIDAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for audioDataTimestampIDAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&audioMetaDataAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for audioMetaDataAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&audioMetaDataHistoryIndexAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for audioMetaDataHistoryIndexAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
    status = MPCreateCriticalRegion(&visualItemIdentifierAccessCriticalRegion);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPCreateQueue for visualItemIdentifierAccessCriticalRegion in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
#endif
#if TARGET_OS_WIN
	::InitializeCriticalSection(&downloadQueueSingletonCriticalSection);
	::InitializeCriticalSection(&downloadQueueAccessCriticalSection);
	::InitializeCriticalSection(&downloadQueueAddRequestCriticalSection);
	::InitializeCriticalSection(&downloadQueueStopRequestCriticalSection);
	::InitializeCriticalSection(&notificationQueueSingletonCriticalSection);
	::InitializeCriticalSection(&notificationQueueAccessCriticalSection);
	::InitializeCriticalSection(&inputNotificationQueueAccessCriticalSection);
	::InitializeCriticalSection(&timeStoreSingletonCriticalSection);
	::InitializeCriticalSection(&timeStoreAccessCriticalSection);
    ::InitializeCriticalSection(&currentTimeAccessCriticalSection);
    ::InitializeCriticalSection(&appProcAccessCriticalSection);
    ::InitializeCriticalSection(&appCookieAccessCriticalSection);
    ::InitializeCriticalSection(&appVersionAccessCriticalSection);
    ::InitializeCriticalSection(&visualizerAPIVersionAccessCriticalSection);
    ::InitializeCriticalSection(&trackPlayPositionAccessCriticalSection);
    ::InitializeCriticalSection(&audioDataTimestampIDAccessCriticalSection);
    ::InitializeCriticalSection(&audioMetaDataAccessCriticalSection);
    ::InitializeCriticalSection(&audioMetaDataHistoryIndexAccessCriticalSection);
    ::InitializeCriticalSection(&visualItemIdentifierAccessCriticalSection);
#endif
}


VisualThreadingManager::~VisualThreadingManager() {
	for (ThreadVectorIterator it = threadVector.begin(); it != threadVector.end(); it++) {
		delete (*it).threadIdentifier;
		(*it).threadIdentifier = NULL;
	}
#if TARGET_OS_MAC
	OSStatus status = noErr;
	status = MPDeleteQueue(notificationQueue);
    if (status != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPDeleteQueue in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(status), __FILE__, __LINE__, __FUNCTION__);
        writeLog(errStr);
    }
#endif
}


VisualThreadingManager* VisualThreadingManager::getInstance() {
    if (theVisualThreadingManager == NULL) {
		theVisualThreadingManager = new VisualThreadingManager;
		if (theVisualThreadingManager == NULL) {
			char errLog[512];
			sprintf(errLog, "ERR: init VisualThreadingManager failed in file: %s (line: %d) [%s])", __FILE__, __LINE__, __FUNCTION__);
			writeLog(errLog);
		}
	}
	return theVisualThreadingManager;
}


void VisualThreadingManager::dispose() {
	if (theVisualThreadingManager != NULL) {
		theVisualThreadingManager->terminateAllThreads = true;
		VisualItemIdentifier terminateAllThreadsId;
		uint32 elapsedMilliseconds = 0;
		while ((VisualThreadingManager::getNumberOfCurrentlyRunningThreads() > 0) && elapsedMilliseconds < 2000) {
			elapsedMilliseconds = VisualTiming::getElapsedMilliSecsSinceReset(terminateAllThreadsId);
			VisualTiming::update(); // manual update of current timestamp
			// wait
		}
		// aborted after max 2 sec. of wait time
		ThreadVectorIterator it = theVisualThreadingManager->threadVector.begin();
		while (it != theVisualThreadingManager->threadVector.end()) {
#if TARGET_OS_MAC
			OSStatus terminationStatus = noErr;
			OSStatus terminationResult = MPTerminateTask(it->threadId, terminationStatus);
            if (terminationResult != noErr)
            {
                char errLog[512];
                sprintf(errLog, "ERR: MPTerminateTask failed (%s) in file: %s (line: %d) [%s])", GetMacOSStatusErrorString(terminationResult), __FILE__, __LINE__, __FUNCTION__);
                writeLog(errLog);
            }
#endif
#if TARGET_OS_WIN
			DWORD dwExitCode = 0;
			BOOL success = TerminateThread(it->threadId, dwExitCode);
#endif
			it = theVisualThreadingManager->threadVector.erase(it);
		}
		delete theVisualThreadingManager;
		theVisualThreadingManager = NULL;
	}
}


size_t VisualThreadingManager::getNumberOfCurrentlyRunningThreads() {
	theVisualThreadingManager = VisualThreadingManager::getInstance();
	return theVisualThreadingManager->threadVector.size();
}


bool VisualThreadingManager::createThread(ThreadingFuncPtr aThreadingFuncPtr, void* param, VisualThreading::ThreadDidDieCallback callbackAfterThreadDidDie) {
	bool success = false;
	theVisualThreadingManager = VisualThreadingManager::getInstance();
    if (theVisualThreadingManager == NULL)
    {
        return success;
    }
	if (theVisualThreadingManager->terminateAllThreads == true) {
		return success;
	}
	VisualItemIdentifier* threadIdentifier = new VisualItemIdentifier;
#if TARGET_OS_MAC
	MPTaskID threadId;
	ByteCount stackSize = 0;
	void* terminationParameter1 = threadIdentifier;
	void* terminationParameter2 = NULL;
	OSStatus osStatus = MPCreateTask(&(*aThreadingFuncPtr), param, stackSize, theVisualThreadingManager->notificationQueue, terminationParameter1, terminationParameter2, 0, &threadId);
	if (osStatus == noErr)
	{
		success = true;
	}
#endif
#if TARGET_OS_WIN
	LPSECURITY_ATTRIBUTES lpThreadAttributes = NULL;
	SIZE_T stackSize = 0;
	DWORD dwCreationFlags = 0;
	LPDWORD threadId = 0;
	HANDLE hdl = CreateThread(lpThreadAttributes, stackSize, (LPTHREAD_START_ROUTINE)aThreadingFuncPtr, (LPVOID)param, dwCreationFlags, threadId);
	if (hdl)
	{
		success = true;
	}
#endif
	if (success == true) {
		theVisualThreadingManager->addThread(threadId, threadIdentifier, callbackAfterThreadDidDie);
	} else {
		delete threadIdentifier;
	}
	return success;	
}


#if TARGET_OS_MAC
void VisualThreadingManager::addThread(MPTaskID threadId, VisualItemIdentifier* threadIdentifier, VisualThreading::ThreadDidDieCallback callbackAfterThreadDidDie) {
#endif
#if TARGET_OS_WIN
void VisualThreadingManager::addThread(HANDLE threadId, VisualItemIdentifier* threadIdentifier, VisualThreading::ThreadDidDieCallback callbackAfterThreadDidDie) {
#endif
	theVisualThreadingManager = VisualThreadingManager::getInstance();
	ThreadItem threadItem;
	threadItem.threadId = threadId;
	threadItem.threadIdentifier = threadIdentifier;
	threadItem.callbackAfterDeathOfThread = callbackAfterThreadDidDie;
	theVisualThreadingManager->threadVector.push_back(threadItem);
}


void VisualThreadingManager::update() {
    theVisualThreadingManager = VisualThreadingManager::getInstance();
    if (theVisualThreadingManager)
    {
        ThreadVectorIterator it = theVisualThreadingManager->threadVector.begin();
        while (it != theVisualThreadingManager->threadVector.end()) {
#if TARGET_OS_MAC
            void* param1 = (void*)((*it).threadIdentifier);
            void* param2 = NULL;
            void* param3= NULL;
            Duration timeout = kDurationImmediate;
            OSStatus status = MPWaitOnQueue(theVisualThreadingManager->notificationQueue, &param1, &param2, &param3, timeout);
            if (status == noErr) {
#endif
#if TARGET_OS_WIN
            DWORD exitCode = 0;
            BOOL success = ::GetExitCodeThread((*it).threadId, &exitCode);
            if (exitCode != STILL_ACTIVE) {
#endif
                // thread terminated
                // a) call registered callback
                if ((*it).callbackAfterDeathOfThread != NULL) {
                    (*it).callbackAfterDeathOfThread();
                }
                // b) unregister thread from queue
                it = theVisualThreadingManager->threadVector.erase(it);
            } else {
                it++;
            }
        }
    }
}

#if TARGET_OS_MAC
MPCriticalRegionID VisualThreadingManager::getCriticalRegionID(VisualCriticalRegion criticalRegionId) {
	theVisualThreadingManager = VisualThreadingManager::getInstance();
    if (theVisualThreadingManager)
    {
        switch (criticalRegionId)
        {
            case downloadQueueSingletonMutex:
                return theVisualThreadingManager->downloadQueueSingletonCriticalRegion;
                break;
            case downloadQueueAccessMutex:
                return theVisualThreadingManager->downloadQueueAccessCriticalRegion;
                break;
            case addRequestDownloadQueueAccess:
                return theVisualThreadingManager->downloadQueueAddRequestCriticalRegion;
                break;
            case stopRequestDownloadQueueAccess:
                return theVisualThreadingManager->downloadQueueStopRequestCriticalRegion;
                break;
            case notificationQueueSingletonMutex:
                return theVisualThreadingManager->notificationQueueSingletonCriticalRegion;
                break;
            case notificationQueueAccessMutex:
                return theVisualThreadingManager->notificationQueueAccessCriticalRegion;
                break;
            case inputNotificationQueueAccessMutex:
                return theVisualThreadingManager->inputNotificationQueueAccessCriticalRegion;
                break;
            case timeStoreSingletonMutex:
                return theVisualThreadingManager->timeStoreSingletonCriticalRegion;
                break;
            case timeStoreAccessMutex:
                return theVisualThreadingManager->timeStoreAccessCriticalRegion;
                break;
            case currentTimeAccessMutex:
                return theVisualThreadingManager->currentTimeAccessCriticalRegion;
                break;
            case appProcAccessMutex:
                return theVisualThreadingManager->appProcAccessCriticalRegion;
                break;
            case appCookieAccessMutex:
                return theVisualThreadingManager->appCookieAccessCriticalRegion;
                break;
            case appVersionAccessMutex:
                return theVisualThreadingManager->appVersionAccessCriticalRegion;
                break;
            case visualizerAPIVersionAccessMutex:
                return theVisualThreadingManager->visualizerAPIVersionAccessCriticalRegion;
                break;
            case trackPlayPositionAccessMutex:
                return theVisualThreadingManager->trackPlayPositionAccessCriticalRegion;
                break;
            case audioDataTimestampIDAccessMutex:
                return theVisualThreadingManager->audioDataTimestampIDAccessCriticalRegion;
                break;
            case audioMetaDataAccessMutex:
                return theVisualThreadingManager->audioMetaDataAccessCriticalRegion;
                break;
            case audioMetaDataHistoryIndexAccessMutex:
                return theVisualThreadingManager->audioMetaDataHistoryIndexAccessCriticalRegion;
                break;
            case visualItemIdentifierAccessMutex:
                return theVisualThreadingManager->visualItemIdentifierAccessCriticalRegion;
                break;
            default:
                return NULL;
        }
    }
	return NULL;
}
#endif


#if TARGET_OS_WIN
LPCRITICAL_SECTION VisualThreadingManager::getCriticalSectionPointer(VisualCriticalRegion criticalRegionId) {
	theVisualThreadingManager = VisualThreadingManager::getInstance();
	switch (criticalRegionId)
	{
		case downloadQueueSingletonMutex:
			return &(theVisualThreadingManager->downloadQueueSingletonCriticalSection);
			break;
		case downloadQueueAccessMutex:
			return &(theVisualThreadingManager->downloadQueueAccessCriticalSection);
			break;
		case addRequestDownloadQueueAccess:
			return &(theVisualThreadingManager->downloadQueueAddRequestCriticalSection);
			break;
		case stopRequestDownloadQueueAccess:
			return &(theVisualThreadingManager->downloadQueueStopRequestCriticalSection);
			break;
		case notificationQueueSingletonMutex:
			return &(theVisualThreadingManager->notificationQueueSingletonCriticalSection);
			break;
		case notificationQueueAccessMutex:
			return &(theVisualThreadingManager->notificationQueueAccessCriticalSection);
			break;
		case inputNotificationQueueAccessMutex:
			return &(theVisualThreadingManager->inputNotificationQueueAccessCriticalSection);
			break;
		case timeStoreSingletonMutex:
			return &(theVisualThreadingManager->timeStoreSingletonCriticalSection);
			break;
		case timeStoreAccessMutex:
			return &(theVisualThreadingManager->timeStoreAccessCriticalSection);
			break;
        case currentTimeAccessMutex:
			return &(theVisualThreadingManager->currentTimeAccessCriticalSection);
			break;
        case appProcAccessMutex:
			return &(theVisualThreadingManager->appProcAccessCriticalSection);
			break;
        case appCookieAccessMutex:
			return &(theVisualThreadingManager->appCookieAccessCriticalSection);
			break;
        case appVersionAccessMutex:
			return &(theVisualThreadingManager->appVersionAccessCriticalSection);
			break;
        case visualizerAPIVersionAccessMutex:
			return &(theVisualThreadingManager->visualizerAPIVersionAccessCriticalSection);
			break;
        case trackPlayPositionAccessMutex:
			return &(theVisualThreadingManager->trackPlayPositionAccessCriticalSection);
			break;
        case audioDataTimestampIDAccessMutex:
			return &(theVisualThreadingManager->audioDataTimestampIDAccessCriticalSection);
			break;
        case audioMetaDataAccessMutex:
			return &(theVisualThreadingManager->audioMetaDataAccessCriticalSection);
			break;
        case audioMetaDataHistoryIndexAccessMutex:
			return &(theVisualThreadingManager->audioMetaDataHistoryIndexAccessCriticalSection);
			break;
        case visualItemIdentifierAccessMutex:
			return &(theVisualThreadingManager->visualItemIdentifierAccessCriticalSection);
			break;
		default:
			return NULL;
	}
	return NULL;
}
#endif

    
void VisualThreadingManager::enterCriticalSection(VisualCriticalRegion aCriticalSectionId)
{
#if TARGET_OS_MAC
    MPCriticalRegionID aCriticalRegion = VisualThreadingManager::getCriticalRegionID(aCriticalSectionId);
    OSStatus aStatus = ::MPEnterCriticalRegion(aCriticalRegion, kDurationForever);
    if (aStatus != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPEnterCriticalRegion", GetMacOSStatusErrorString(aStatus));
        writeLog(errStr);
    }
#endif
#if TARGET_OS_WIN
    LPCRITICAL_SECTION aCriticalSection = VisualThreadingManager::getCriticalSectionPointer(aCriticalSectionId);
    ::EnterCriticalSection(aCriticalSection);
#endif
}


void VisualThreadingManager::exitCriticalSection(VisualCriticalRegion aCriticalSectionId)
{
#if TARGET_OS_MAC
    MPCriticalRegionID aCriticalRegion = VisualThreadingManager::getCriticalRegionID(aCriticalSectionId);
    OSStatus aStatus = ::MPExitCriticalRegion(aCriticalRegion);
    if (aStatus != noErr)
    {
        char errStr[256];
        sprintf(errStr, "Err: Failure (%s) on MPExitCriticalRegion", GetMacOSStatusErrorString(aStatus));
        writeLog(errStr);
    }
#endif
#if TARGET_OS_WIN
    LPCRITICAL_SECTION aCriticalSection = VisualThreadingManager::getCriticalSectionPointer(aCriticalSectionId);
    ::LeaveCriticalSection(aCriticalSection);
#endif
}
